# Write your code here :-)
from tilt import Tilt
from orb import *
import imu
import board
import neopixel
import time, random

RED = (255,0,0)
BLACK = (0,0,0)

# Update to match the pin connected to your NeoPixels
pixel_pin = board.D4
# Update to match the number of NeoPixels you have connected
pixel_num = 150

pixels = neopixel.NeoPixel(pixel_pin, pixel_num, brightness=0.4, auto_write=False)
pixels.fill((0,0,0))

def orb5x5():
    width = 5
    gap = 0.25
    X = Matrix.X
    Y = Matrix.Y
    Z = Matrix.Z
    npix = width*width
    maxv = width-1 #largest coordinate value
    #definition of matrix defaults to alternating rows
    #The reference coodrinate frame for the orb is the one in which the
    #XIAO X-axis is along the short board axis, the positive Y axis is along
    #the long axis with positive direction pointed towards the USB connector
    #and positive Z axis is above the top of the board
    front = Matrix([0, -gap, maxv], X, -Z, 0, width)
    right = Matrix([maxv + gap, maxv, maxv], -Z, -Y, 1*npix, width)
    bottom = Matrix([maxv, maxv, -gap], -X, -Y, 2*npix, width)
    left = Matrix([-gap, maxv, 0], Z, -Y, 3*npix, width)
    top = Matrix([0, 0, maxv+gap], Y, X, 4*npix, width)
    back = Matrix([0, maxv+gap, maxv], -Z, X, 5*npix, width)
    orb = Orb(5,[front, right, bottom, left, top, back],
                ["front", "right", "bottom", "left", "top", "back"], pixels)
    return orb

#Create orb object
orb=orb5x5()

#Helper function to see the sequence of LEDs in each matrix
def do_pixel(matrix=None, color=RED):
    start = 0
    stop = pixel_num
    if matrix != None:
        pix_per_matrix = orb.width*orb.width
        start = matrix*pix_per_matrix
        stop = (matrix + 1)*pix_per_matrix
    for i in range(start, stop):
        pixels[i] = color
        pixels.show()
        pixels[i] = BLACK
        time.sleep(0.1)

#Helper function to fill one section of the matrix at a time
def fill_matrix(index, color=(255,0,0), axis=0):
    pix_per_matrix = orb.width*orb.width
    start = index*pix_per_matrix
    end = start + pix_per_matrix
    comparator = lambda x : True
    if axis > 0:
        comparator = lambda x : x > 0
    elif axis < 0:
        comparator = lambda x : x < 0
    for i in range(start, end):
        if axis == 0:
            pixels[i] = color
        elif comparator(orb.cart[i][abs(axis)-1]):
            pixels[i] = color
        else:
            pixels[i] = BLACK


def do_matrix(color=RED):
    for i in range(6):
        name = orb.matrix_name(i)
        orb.fill_matrix(i,color)
        time.sleep(.5)
        orb.fill_matrix(i, BLACK)


imu = imu.IMU()
tilt = Tilt(orb.pixels, orb.cart, imu)

while True:
    tilt.animate()
